import * as t from 'io-ts';
import { isRight } from 'fp-ts/These';
import { either } from 'fp-ts/Either';
import { PathReporter } from 'io-ts/lib/PathReporter';
import * as iotsTypes from 'io-ts-types';
export function validate(schema, params, errorName) {
  const validationResult = schema.decode(params);

  if (!isRight(validationResult)) {
    const messages = PathReporter.report(validationResult);
    const error = new Error(messages.join('\r\n'));

    if (errorName != null) {
      error.name = errorName;
    }

    throw error;
  }

  return params;
}
export function getInitialReplicationState() {
  return {
    status: 'notStarted',
    statusData: null,
    iterator: null,
    paused: false,
    successEvent: null,
    locked: false
  };
}
const EventFilterCommonSchema = t.intersection([t.type({
  limit: t.Int
}), t.partial({
  eventTypes: t.union([t.array(t.string), t.null]),
  aggregateIds: t.union([t.array(t.string), t.null]),
  eventsSizeLimit: t.Int
})]);
const TimestampFilterFieldsSchema = t.partial({
  startTime: t.Int,
  finishTime: t.Int
});
const TimestampFilterSchema = t.intersection([EventFilterCommonSchema, TimestampFilterFieldsSchema]);
const CursorFilterFieldsSchema = t.type({
  cursor: t.union([t.string, t.null])
});
const CursorFilterSchema = t.intersection([EventFilterCommonSchema, CursorFilterFieldsSchema]);
const EventFilterSchemaSimple = t.intersection([EventFilterCommonSchema, TimestampFilterFieldsSchema, t.partial(CursorFilterFieldsSchema.props)]);
export const EventFilterSchema = new t.Type('EventFilterSchema', EventFilterSchemaSimple.is, (u, c) => either.chain(EventFilterSchemaSimple.validate(u, c), filter => {
  if (isCursorFilter(filter)) {
    if (isTimestampFilter(filter)) {
      return t.failure(u, c, 'startTime and finishTime are not allowed in cursor filter');
    }

    return t.success(filter);
  } else if (isTimestampFilter(filter)) {
    if (filter.startTime !== undefined && filter.finishTime !== undefined) {
      if (filter.startTime > filter.finishTime) {
        return t.failure(u, c, `Event filter start time cannot be later than finishTime`);
      }
    }

    return t.success(filter);
  } else {
    return t.failure(u, c, 'Cursor or at least one of startTime or finishTime should be defined');
  }
}), t.identity);
export function isTimestampFilter(filter) {
  return filter.startTime != null || filter.finishTime != null;
}
export function isCursorFilter(filter) {
  return filter.cursor !== undefined;
} // eslint-disable-next-line @typescript-eslint/no-unused-vars

export const AdapterConfigSchema = t.partial({
  snapshotBucketSize: t.Int
});
export const AdapterTableNamesSchema = t.partial({
  secretsTableName: iotsTypes.NonEmptyString,
  eventsTableName: iotsTypes.NonEmptyString,
  snapshotsTableName: iotsTypes.NonEmptyString,
  subscribersTableName: iotsTypes.NonEmptyString
});
//# sourceMappingURL=types.js.map