import { getLog } from './get-log';
import { AlreadyDisposedError } from './errors';

const wrapDispose = (pool, dispose) => async () => {
  if (pool.disposed) {
    throw new AlreadyDisposedError();
  }

  pool.disposed = true;
  const log = getLog('dispose');

  if (!pool.isConnected) {
    log.debug('event store is not yet connected, no need for disconnecting');
    return;
  }

  if (pool.getConnectPromise) await pool.getConnectPromise();
  await dispose(pool);
};

export default wrapDispose;
//# sourceMappingURL=wrap-dispose.js.map