"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _getLog = require("./get-log");

var _wrapMethod = _interopRequireDefault(require("./wrap-method"));

var _wrapEventFilter = _interopRequireDefault(require("./wrap-event-filter"));

var _wrapDispose = _interopRequireDefault(require("./wrap-dispose"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const emptyFunction = async pool => {
  return;
};

const getSecretsManager = pool => {
  return {
    deleteSecret: pool.deleteSecret,
    getSecret: pool.getSecret,
    setSecret: pool.setSecret
  };
};

const createAdapter = ({
  maybeThrowResourceError,
  validateEventFilter,
  loadEvents,
  importEventsStream,
  exportEventsStream,
  incrementalImport,
  getNextCursor,
  importSecretsStream,
  exportSecretsStream,
  init,
  drop,
  gatherSecretsFromEvents,
  getEventLoader
}, {
  connect,
  dispose,
  loadEventsByCursor,
  loadEventsByTimestamp,
  ensureEventSubscriber,
  removeEventSubscriber,
  getEventSubscribers,
  getLatestEvent,
  saveEvent,
  initEvents,
  initSecrets,
  initFinal,
  dropEvents,
  dropSecrets,
  dropFinal,
  injectEvent,
  injectEvents,
  freeze,
  unfreeze,
  shapeEvent,
  loadSnapshot,
  saveSnapshot,
  dropSnapshot,
  beginIncrementalImport,
  commitIncrementalImport,
  rollbackIncrementalImport,
  pushIncrementalImport,
  getSecret,
  setSecret,
  deleteSecret,
  loadSecrets,
  injectSecret,
  replicateEvents,
  replicateSecrets,
  setReplicationPaused,
  setReplicationStatus,
  getReplicationState,
  resetReplication,
  setReplicationLock,
  getCursorUntilEventTypes,
  describe,
  establishTimeLimit,
  getEventLoaderNative
}, connectionDependencies, options, prepare) => {
  const log = (0, _getLog.getLog)(`createAdapter`);
  const config = { ...options
  };
  let bucketSize = 100;
  const {
    snapshotBucketSize
  } = config;

  if (snapshotBucketSize !== undefined && Number.isSafeInteger(snapshotBucketSize) && snapshotBucketSize > 0) {
    bucketSize = snapshotBucketSize;
    log.debug(`snapshot bucket size explicitly set to ${bucketSize}`);
  } else {
    log.debug(`snapshot bucket size defaulted to ${bucketSize}`);
  }

  const createGetConnectPromise = () => {
    let p;
    return () => {
      if (p === undefined) {
        p = connect(adapterPool, connectionDependencies, config);
      }

      return p;
    };
  };

  const primalProps = {
    disposed: false,
    validateEventFilter,
    isConnected: false,
    maybeThrowResourceError,
    bucketSize,
    getNextCursor: getNextCursor.bind(null),
    counters: new Map(),
    createGetConnectPromise: createGetConnectPromise,
    getConnectPromise: createGetConnectPromise()
  };
  const emptyProps = {};
  const adapterPool = { ...primalProps,
    ...emptyProps
  };

  if (prepare !== undefined) {
    prepare(adapterPool, config, connectionDependencies);
  }

  const connectedProps = {
    injectEvent: (0, _wrapMethod.default)(adapterPool, injectEvent),
    injectEvents: (0, _wrapMethod.default)(adapterPool, injectEvents),
    injectSecret: (0, _wrapMethod.default)(adapterPool, injectSecret),
    loadEventsByCursor: (0, _wrapMethod.default)(adapterPool, loadEventsByCursor),
    loadEventsByTimestamp: (0, _wrapMethod.default)(adapterPool, loadEventsByTimestamp),
    deleteSecret: (0, _wrapMethod.default)(adapterPool, deleteSecret),
    getSecret: (0, _wrapMethod.default)(adapterPool, getSecret),
    setSecret: (0, _wrapMethod.default)(adapterPool, setSecret),
    initEvents: (0, _wrapMethod.default)(adapterPool, initEvents),
    initSecrets: (0, _wrapMethod.default)(adapterPool, initSecrets),
    initFinal: (0, _wrapMethod.default)(adapterPool, initFinal),
    dropEvents: (0, _wrapMethod.default)(adapterPool, dropEvents),
    dropSecrets: (0, _wrapMethod.default)(adapterPool, dropSecrets),
    dropFinal: (0, _wrapMethod.default)(adapterPool, dropFinal),
    waitConnect: (0, _wrapMethod.default)(adapterPool, emptyFunction),
    shapeEvent,
    getEventLoaderNative: getEventLoaderNative ? async filter => {
      return getEventLoaderNative(adapterPool, filter);
    } : getEventLoaderNative
  };
  Object.assign(adapterPool, connectedProps);
  const adapter = {
    loadEvents: (0, _wrapMethod.default)(adapterPool, (0, _wrapEventFilter.default)(loadEvents)),
    importEvents: importEventsStream.bind(null, adapterPool),
    exportEvents: exportEventsStream.bind(null, adapterPool),
    getLatestEvent: (0, _wrapMethod.default)(adapterPool, getLatestEvent),
    saveEvent: (0, _wrapMethod.default)(adapterPool, saveEvent),
    init: (0, _wrapMethod.default)(adapterPool, init),
    drop: (0, _wrapMethod.default)(adapterPool, drop),
    dispose: (0, _wrapDispose.default)(adapterPool, dispose),
    freeze: (0, _wrapMethod.default)(adapterPool, freeze),
    unfreeze: (0, _wrapMethod.default)(adapterPool, unfreeze),
    getNextCursor: getNextCursor.bind(null),
    getSecretsManager: async () => {
      return getSecretsManager(adapterPool);
    },
    loadSnapshot: (0, _wrapMethod.default)(adapterPool, loadSnapshot),
    saveSnapshot: (0, _wrapMethod.default)(adapterPool, saveSnapshot),
    dropSnapshot: (0, _wrapMethod.default)(adapterPool, dropSnapshot),
    pushIncrementalImport: (0, _wrapMethod.default)(adapterPool, pushIncrementalImport),
    beginIncrementalImport: (0, _wrapMethod.default)(adapterPool, beginIncrementalImport),
    commitIncrementalImport: (0, _wrapMethod.default)(adapterPool, commitIncrementalImport),
    rollbackIncrementalImport: (0, _wrapMethod.default)(adapterPool, rollbackIncrementalImport),
    incrementalImport: (0, _wrapMethod.default)(adapterPool, incrementalImport),
    loadSecrets: (0, _wrapMethod.default)(adapterPool, loadSecrets),
    importSecrets: importSecretsStream.bind(null, adapterPool),
    exportSecrets: exportSecretsStream.bind(null, adapterPool),
    ensureEventSubscriber: (0, _wrapMethod.default)(adapterPool, ensureEventSubscriber),
    removeEventSubscriber: (0, _wrapMethod.default)(adapterPool, removeEventSubscriber),
    getEventSubscribers: (0, _wrapMethod.default)(adapterPool, getEventSubscribers),
    gatherSecretsFromEvents: (0, _wrapMethod.default)(adapterPool, gatherSecretsFromEvents),
    replicateEvents: (0, _wrapMethod.default)(adapterPool, replicateEvents),
    replicateSecrets: (0, _wrapMethod.default)(adapterPool, replicateSecrets),
    setReplicationPaused: (0, _wrapMethod.default)(adapterPool, setReplicationPaused),
    setReplicationStatus: (0, _wrapMethod.default)(adapterPool, setReplicationStatus),
    getReplicationState: (0, _wrapMethod.default)(adapterPool, getReplicationState),
    resetReplication: (0, _wrapMethod.default)(adapterPool, resetReplication),
    setReplicationLock: (0, _wrapMethod.default)(adapterPool, setReplicationLock),
    getCursorUntilEventTypes: (0, _wrapMethod.default)(adapterPool, getCursorUntilEventTypes),
    describe: (0, _wrapMethod.default)(adapterPool, describe),
    establishTimeLimit: establishTimeLimit === undefined ? f => {
      return;
    } : establishTimeLimit.bind(null, adapterPool),
    getEventLoader: async (filter, options) => {
      return getEventLoader(adapterPool, filter, options);
    }
  };
  Object.assign(adapterPool, adapter);
  return Object.freeze(adapter);
};

var _default = createAdapter;
exports.default = _default;
//# sourceMappingURL=create-adapter.js.map