"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _stream = require("stream");

var _constants = require("./constants");

var _frozenErrors = require("./frozen-errors");

async function startProcessSecrets({
  pool,
  maintenanceMode
}) {
  if (maintenanceMode === _constants.MAINTENANCE_MODE_AUTO) {
    try {
      await pool.freeze();
    } catch (error) {
      if (!_frozenErrors.AlreadyFrozenError.is(error)) {
        throw error;
      }
    }
  }
}

async function endProcessSecrets({
  pool,
  maintenanceMode
}) {
  if (maintenanceMode === _constants.MAINTENANCE_MODE_AUTO) {
    try {
      await pool.unfreeze();
    } catch (error) {
      if (!_frozenErrors.AlreadyUnfrozenError.is(error)) {
        throw error;
      }
    }
  }
}

async function* generator(context) {
  const {
    pool
  } = context;
  await pool.waitConnect();
  await startProcessSecrets(context);

  while (true) {
    const {
      secrets,
      idx
    } = await pool.loadSecrets({
      idx: context.idx,
      limit: _constants.BATCH_SIZE
    });

    for (const secret of secrets) {
      const chunk = Buffer.from(JSON.stringify(secret) + '\n', 'utf8');
      yield chunk;
      context.idx = idx;
    }

    if (secrets.length === 0) {
      await endProcessSecrets(context);
      return;
    }
  }
}

const exportSecretsStream = (pool, {
  idx = null,
  maintenanceMode = _constants.MAINTENANCE_MODE_AUTO
} = {}) => {
  if (pool.loadSecrets === undefined) throw new Error('loadSecrets is not defined for this adapter');
  const context = {
    pool,
    idx,
    maintenanceMode
  };

  const stream = _stream.Readable.from(generator(context));

  Object.defineProperty(stream, 'idx', {
    get() {
      return context.idx;
    }

  });
  return stream;
};

var _default = exportSecretsStream;
exports.default = _default;
//# sourceMappingURL=export-secrets.js.map