"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _getLog = require("./get-log");

const snapshotTrigger = async (state, snapshotKey, content, updateCallback) => {
  const log = (0, _getLog.getLog)(`snapshotTrigger`);
  const {
    bucketSize,
    counters
  } = state;
  log.verbose(`snapshotKey: ${snapshotKey}`);
  log.verbose(`bucketSize: ${bucketSize}`);

  if (snapshotKey == null || snapshotKey.constructor !== String) {
    const error = new Error('Snapshot key must be a string');
    log.error(error.message);
    throw error;
  }

  if (content == null || content.constructor !== String) {
    const error = new Error('Snapshot content must be a string');
    log.error(error.message);
    throw error;
  }

  const snapshotIndex = counters.get(snapshotKey);

  if (snapshotIndex === undefined) {
    counters.set(snapshotKey, 1);
  } else {
    counters.set(snapshotKey, snapshotIndex + 1);
  }

  const pendingSnapshotIndex = counters.get(snapshotKey);

  if (pendingSnapshotIndex !== undefined && pendingSnapshotIndex < bucketSize) {
    log.verbose(`pending snapshot: (${counters.get(snapshotKey)}/${bucketSize})`);
    return Promise.resolve();
  }

  counters.set(snapshotKey, 0);
  log.debug(`updating snapshot ${snapshotKey}`);
  return updateCallback();
};

var _default = snapshotTrigger;
exports.default = _default;
//# sourceMappingURL=snapshot-trigger.js.map