/// <reference types="node" />
import type { SecretsManager, Event, StoredEvent, EventThreadData, StoredEventPointer, StoredEventBatchPointer, Eventstore as CoreEventstore, InputCursor, Cursor, SecretRecord, OldSecretRecord, OldEvent, ReplicationStatus, ReplicationState } from '@resolve-js/core';
import stream from 'stream';
import { MAINTENANCE_MODE_AUTO, MAINTENANCE_MODE_MANUAL } from './constants';
import * as t from 'io-ts';
import * as iotsTypes from 'io-ts-types';
export type { StoredEvent, EventThreadData, StoredEventPointer, StoredEventBatchPointer, InputCursor, Cursor, };
export declare function validate<T extends t.Type<any>>(schema: T, params: any, errorName?: string): t.TypeOf<T>;
declare type DefinedType<T extends any> = undefined extends Extract<T, undefined> ? Exclude<T, undefined> : T;
export declare type UnbrandProps<T extends any> = {
    [Property in keyof T]: DefinedType<T[Property]> extends string & t.Brand<infer B> ? string | Extract<T[Property], undefined> : DefinedType<T[Property]> extends t.Branded<infer S, infer B> ? S | Extract<T[Property], undefined> : T[Property];
};
export declare type InputEvent = Event;
export declare type VersionlessEvent = Omit<InputEvent, 'aggregateVersion'>;
export type { SecretRecord, OldSecretRecord, OldEvent };
export type { ReplicationState, ReplicationStatus };
export declare function getInitialReplicationState(): ReplicationState;
export declare type CheckForResourceError = (errors: Error[]) => void;
declare type DeleteSecret = SecretsManager['deleteSecret'];
declare type GetSecret = SecretsManager['getSecret'];
declare type SetSecret = SecretsManager['setSecret'];
declare type ShapeEvent = (event: any, additionalFields?: any) => StoredEvent;
export declare type ValidateEventFilter = (filter: EventFilter) => void;
declare const TimestampFilterSchema: t.IntersectionC<[t.IntersectionC<[t.TypeC<{
    limit: t.BrandC<t.NumberC, t.IntBrand>;
}>, t.PartialC<{
    eventTypes: t.UnionC<[t.ArrayC<t.StringC>, t.NullC]>;
    aggregateIds: t.UnionC<[t.ArrayC<t.StringC>, t.NullC]>;
    eventsSizeLimit: t.BrandC<t.NumberC, t.IntBrand>;
}>]>, t.PartialC<{
    startTime: t.BrandC<t.NumberC, t.IntBrand>;
    finishTime: t.BrandC<t.NumberC, t.IntBrand>;
}>]>;
declare type TimestampFilterChecked = t.TypeOf<typeof TimestampFilterSchema>;
export declare type TimestampFilter = UnbrandProps<TimestampFilterChecked>;
declare const CursorFilterSchema: t.IntersectionC<[t.IntersectionC<[t.TypeC<{
    limit: t.BrandC<t.NumberC, t.IntBrand>;
}>, t.PartialC<{
    eventTypes: t.UnionC<[t.ArrayC<t.StringC>, t.NullC]>;
    aggregateIds: t.UnionC<[t.ArrayC<t.StringC>, t.NullC]>;
    eventsSizeLimit: t.BrandC<t.NumberC, t.IntBrand>;
}>]>, t.TypeC<{
    cursor: t.UnionC<[t.StringC, t.NullC]>;
}>]>;
declare type CursorFilterChecked = t.TypeOf<typeof CursorFilterSchema>;
export declare type CursorFilter = UnbrandProps<CursorFilterChecked>;
declare const EventFilterSchemaSimple: t.IntersectionC<[t.IntersectionC<[t.TypeC<{
    limit: t.BrandC<t.NumberC, t.IntBrand>;
}>, t.PartialC<{
    eventTypes: t.UnionC<[t.ArrayC<t.StringC>, t.NullC]>;
    aggregateIds: t.UnionC<[t.ArrayC<t.StringC>, t.NullC]>;
    eventsSizeLimit: t.BrandC<t.NumberC, t.IntBrand>;
}>]>, t.PartialC<{
    startTime: t.BrandC<t.NumberC, t.IntBrand>;
    finishTime: t.BrandC<t.NumberC, t.IntBrand>;
}>, t.PartialC<{
    cursor: t.UnionC<[t.StringC, t.NullC]>;
}>]>;
export declare const EventFilterSchema: t.Type<UnbrandProps<{
    limit: t.Branded<number, t.IntBrand>;
} & {
    eventTypes?: string[] | null | undefined;
    aggregateIds?: string[] | null | undefined;
    eventsSizeLimit?: t.Branded<number, t.IntBrand> | undefined;
} & {
    startTime?: t.Branded<number, t.IntBrand> | undefined;
    finishTime?: t.Branded<number, t.IntBrand> | undefined;
} & {
    cursor?: string | null | undefined;
}>, UnbrandProps<{
    limit: t.Branded<number, t.IntBrand>;
} & {
    eventTypes?: string[] | null | undefined;
    aggregateIds?: string[] | null | undefined;
    eventsSizeLimit?: t.Branded<number, t.IntBrand> | undefined;
} & {
    startTime?: t.Branded<number, t.IntBrand> | undefined;
    finishTime?: t.Branded<number, t.IntBrand> | undefined;
} & {
    cursor?: string | null | undefined;
}>, unknown>;
declare type EventFilterChecked = t.TypeOf<typeof EventFilterSchemaSimple>;
export declare type EventFilter = UnbrandProps<EventFilterChecked>;
export declare type LatestEventFilter = Omit<EventFilter, 'limit' | 'eventsSizeLimit'>;
export declare type EventLoaderFilter = Omit<CursorFilter, 'limit' | 'eventsSizeLimit'>;
export declare type SecretFilter = {
    idx?: SecretRecord['idx'] | null;
    skip?: number;
    limit: number;
    ids?: Array<SecretRecord['id']> | null;
    includeDeleted?: boolean;
};
export declare type SecretsWithIdx = {
    idx: SecretRecord['idx'] | null;
    secrets: SecretRecord[];
};
export declare type GatheredSecrets = {
    existingSecrets: SecretRecord[];
    deletedSecrets: Array<SecretRecord['id']>;
};
export declare function isTimestampFilter(filter: EventFilter): filter is TimestampFilter;
export declare function isCursorFilter(filter: EventFilter): filter is CursorFilter;
export declare type RemoveFirstType<T extends any[]> = T extends [infer _, ...infer R] ? R : never;
export declare type PromiseResultType<T extends any> = T extends Promise<infer R> ? R : T;
export declare type PoolMethod<ConnectedProps extends AdapterPoolConnectedProps, M extends (...args: any) => any> = (pool: AdapterPoolConnected<ConnectedProps>, ...args: Parameters<M>) => ReturnType<M>;
export declare type UnconnectedPoolMethod<ConnectedProps extends AdapterPoolConnectedProps, M extends (...args: any) => any> = (pool: AdapterPoolPossiblyUnconnected<ConnectedProps>, ...args: Parameters<M>) => ReturnType<M>;
export declare const AdapterConfigSchema: t.PartialC<{
    snapshotBucketSize: t.BrandC<t.NumberC, t.IntBrand>;
}>;
declare type AdapterConfigChecked = t.TypeOf<typeof AdapterConfigSchema>;
export declare type AdapterConfig = UnbrandProps<AdapterConfigChecked>;
export declare const AdapterTableNamesSchema: t.PartialC<{
    secretsTableName: iotsTypes.NonEmptyStringC;
    eventsTableName: iotsTypes.NonEmptyStringC;
    snapshotsTableName: iotsTypes.NonEmptyStringC;
    subscribersTableName: iotsTypes.NonEmptyStringC;
}>;
declare type AdapterTableNamesChecked = t.TypeOf<typeof AdapterTableNamesSchema>;
export declare type AdapterTableNames = UnbrandProps<AdapterTableNamesChecked>;
export declare type AdapterTableNamesProps = Required<AdapterTableNames>;
export declare type AdapterPoolPrimalProps = {
    disposed: boolean;
    validateEventFilter: ValidateEventFilter;
    isConnected: boolean;
    createGetConnectPromise: () => () => Promise<void>;
    getConnectPromise: () => Promise<void>;
    maybeThrowResourceError: CheckForResourceError;
    bucketSize: number;
    counters: Map<string, number>;
    getNextCursor: CoreEventstore['getNextCursor'];
    getVacantTimeInMillis?: () => number;
};
export declare type AdapterPoolPrivateConnectedProps = {
    injectEvent: (event: StoredEvent) => Promise<void>;
    injectEvents: (events: StoredEvent[]) => Promise<void>;
    injectSecret?: (secretRecord: SecretRecord) => Promise<void>;
    loadEventsByTimestamp: (filter: TimestampFilter) => Promise<StoredEventBatchPointer>;
    loadEventsByCursor: (filter: CursorFilter) => Promise<StoredEventBatchPointer>;
    deleteSecret: DeleteSecret;
    getSecret: GetSecret;
    setSecret: SetSecret;
    waitConnect: () => Promise<void>;
    shapeEvent: ShapeEvent;
    initEvents: () => Promise<any[]>;
    initSecrets: () => Promise<any[]>;
    initFinal: () => Promise<any[]>;
    dropEvents: () => Promise<any[]>;
    dropSecrets: () => Promise<any[]>;
    dropFinal: () => Promise<any[]>;
    getEventLoaderNative?: (filter: EventLoaderFilter) => Promise<EventLoader>;
};
export declare type AdapterPoolConnectedProps = Adapter & AdapterPoolPrivateConnectedProps;
export declare type AdapterPoolPossiblyUnconnected<ConnectedProps extends AdapterPoolConnectedProps> = AdapterPoolPrimalProps & Partial<ConnectedProps>;
export declare type AdapterPoolConnected<ConnectedProps extends AdapterPoolConnectedProps> = AdapterPoolPrimalProps & ConnectedProps;
export declare type MAINTENANCE_MODE = typeof MAINTENANCE_MODE_AUTO | typeof MAINTENANCE_MODE_MANUAL;
export declare type ImportOptions = {
    byteOffset: number;
    maintenanceMode: MAINTENANCE_MODE;
};
export declare type ImportEventsStream = stream.Writable & {
    readonly byteOffset: number;
    readonly savedEventsCount: number;
};
export declare type ExportOptions = {
    cursor: InputCursor;
    maintenanceMode: MAINTENANCE_MODE;
    bufferSize: number;
};
export declare type ExportEventsStream = stream.Readable & {
    readonly cursor: InputCursor;
    readonly isBufferOverflow: boolean;
    readonly isEnd: boolean;
};
export declare type ImportSecretsOptions = {
    maintenanceMode: MAINTENANCE_MODE;
};
export declare type ExportSecretsOptions = {
    idx: number | null;
    maintenanceMode: MAINTENANCE_MODE;
};
export interface CommonAdapterFunctions<ConnectedProps extends AdapterPoolConnectedProps> {
    maybeThrowResourceError: CheckForResourceError;
    validateEventFilter: ValidateEventFilter;
    loadEvents: PoolMethod<ConnectedProps, Adapter['loadEvents']>;
    importEventsStream: UnconnectedPoolMethod<ConnectedProps, Adapter['importEvents']>;
    exportEventsStream: UnconnectedPoolMethod<ConnectedProps, Adapter['exportEvents']>;
    incrementalImport: PoolMethod<ConnectedProps, Adapter['incrementalImport']>;
    getNextCursor: CoreEventstore['getNextCursor'];
    importSecretsStream: UnconnectedPoolMethod<ConnectedProps, Adapter['importSecrets']>;
    exportSecretsStream: UnconnectedPoolMethod<ConnectedProps, Adapter['exportSecrets']>;
    init: PoolMethod<ConnectedProps, Adapter['init']>;
    drop: PoolMethod<ConnectedProps, Adapter['drop']>;
    gatherSecretsFromEvents: PoolMethod<ConnectedProps, Adapter['gatherSecretsFromEvents']>;
    getEventLoader: PoolMethod<ConnectedProps, Adapter['getEventLoader']>;
}
export interface AdapterFunctions<ConnectedProps extends AdapterPoolConnectedProps, ConnectionDependencies extends any, Config extends AdapterConfig> {
    beginIncrementalImport: PoolMethod<ConnectedProps, Adapter['beginIncrementalImport']>;
    commitIncrementalImport: PoolMethod<ConnectedProps, Adapter['commitIncrementalImport']>;
    connect: (pool: AdapterPoolPossiblyUnconnected<ConnectedProps>, connectionDependencies: ConnectionDependencies, config: Config) => Promise<void>;
    dispose: PoolMethod<ConnectedProps, Adapter['dispose']>;
    dropSnapshot: PoolMethod<ConnectedProps, Adapter['dropSnapshot']>;
    freeze: PoolMethod<ConnectedProps, Adapter['freeze']>;
    getLatestEvent: PoolMethod<ConnectedProps, Adapter['getLatestEvent']>;
    injectEvent: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['injectEvent']>;
    injectEvents: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['injectEvents']>;
    loadEventsByCursor: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['loadEventsByCursor']>;
    loadEventsByTimestamp: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['loadEventsByTimestamp']>;
    loadSnapshot: PoolMethod<ConnectedProps, Adapter['loadSnapshot']>;
    pushIncrementalImport: PoolMethod<ConnectedProps, Adapter['pushIncrementalImport']>;
    rollbackIncrementalImport: PoolMethod<ConnectedProps, Adapter['rollbackIncrementalImport']>;
    saveEvent: PoolMethod<ConnectedProps, Adapter['saveEvent']>;
    saveSnapshot: PoolMethod<ConnectedProps, Adapter['saveSnapshot']>;
    shapeEvent: ShapeEvent;
    unfreeze: PoolMethod<ConnectedProps, Adapter['unfreeze']>;
    getSecret: PoolMethod<ConnectedProps, GetSecret>;
    setSecret: PoolMethod<ConnectedProps, SetSecret>;
    deleteSecret: PoolMethod<ConnectedProps, DeleteSecret>;
    loadSecrets?: PoolMethod<ConnectedProps, Adapter['loadSecrets']>;
    injectSecret?: PoolMethod<ConnectedProps, NonNullable<AdapterPoolConnectedProps['injectSecret']>>;
    initEvents: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['initEvents']>;
    initSecrets: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['initSecrets']>;
    initFinal: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['initFinal']>;
    dropEvents: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['dropEvents']>;
    dropSecrets: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['dropSecrets']>;
    dropFinal: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['dropFinal']>;
    ensureEventSubscriber: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['ensureEventSubscriber']>;
    removeEventSubscriber: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['removeEventSubscriber']>;
    getEventSubscribers: PoolMethod<ConnectedProps, AdapterPoolConnectedProps['getEventSubscribers']>;
    replicateEvents?: PoolMethod<ConnectedProps, Adapter['replicateEvents']>;
    replicateSecrets?: PoolMethod<ConnectedProps, Adapter['replicateSecrets']>;
    setReplicationStatus?: PoolMethod<ConnectedProps, Adapter['setReplicationStatus']>;
    setReplicationPaused?: PoolMethod<ConnectedProps, Adapter['setReplicationPaused']>;
    getReplicationState?: PoolMethod<ConnectedProps, Adapter['getReplicationState']>;
    resetReplication?: PoolMethod<ConnectedProps, Adapter['resetReplication']>;
    setReplicationLock?: PoolMethod<ConnectedProps, Adapter['setReplicationLock']>;
    getCursorUntilEventTypes?: PoolMethod<ConnectedProps, Adapter['getCursorUntilEventTypes']>;
    describe: PoolMethod<ConnectedProps, Adapter['describe']>;
    establishTimeLimit?: UnconnectedPoolMethod<ConnectedProps, Adapter['establishTimeLimit']>;
    getEventLoaderNative?: PoolMethod<ConnectedProps, NonNullable<AdapterPoolConnectedProps['getEventLoaderNative']>>;
}
export interface EventLoader {
    readonly loadEvents: (limit: number) => Promise<StoredEventBatchPointer>;
    readonly close: () => Promise<void>;
    readonly cursor: InputCursor;
    readonly isNative: boolean;
}
export declare type EventLoaderOptions = {
    preferRegular: boolean;
};
export interface Adapter extends CoreEventstore {
    importEvents: (options?: Partial<ImportOptions>) => ImportEventsStream;
    exportEvents: (options?: Partial<ExportOptions>) => ExportEventsStream;
    getLatestEvent: (filter: LatestEventFilter) => Promise<StoredEvent | null>;
    init: () => Promise<void>;
    drop: () => Promise<void>;
    dispose: () => Promise<void>;
    freeze: () => Promise<void>;
    unfreeze: () => Promise<void>;
    getSecretsManager: () => Promise<SecretsManager>;
    dropSnapshot: (snapshotKey: string) => Promise<void>;
    pushIncrementalImport: (events: VersionlessEvent[], importId: string) => Promise<void>;
    beginIncrementalImport: () => Promise<string>;
    commitIncrementalImport: (importId: string, validateAfterCommit?: any) => Promise<void>;
    rollbackIncrementalImport: () => Promise<void>;
    incrementalImport: (events: VersionlessEvent[]) => Promise<void>;
    loadSecrets: (filter: SecretFilter) => Promise<SecretsWithIdx>;
    importSecrets: (options?: Partial<ImportSecretsOptions>) => stream.Writable;
    exportSecrets: (options?: Partial<ExportSecretsOptions>) => stream.Readable;
    gatherSecretsFromEvents: (events: StoredEvent[]) => Promise<GatheredSecrets>;
    getCursorUntilEventTypes: (cursor: InputCursor, untilEventTypes: Array<InputEvent['type']>) => Promise<string>;
    establishTimeLimit: (getVacantTimeInMillis: () => number) => void;
    getEventLoader: (filter: EventLoaderFilter, options?: EventLoaderOptions) => Promise<EventLoader>;
}
